;+
;
;Procedure: add_leap_seconds
;
;Purpose:
;  Takes a TDAS date and adds leap seconds to the date(s) using the data in the CDF 3.4(or later) leap second config file.
;  Like time_string and time_double, it is vectorized and accepts array inputs of arbitrary dimensions
;
;Inputs: 
;  Dates: double precision seconds since 1970 or date strings.  Can be scalar or array values with any dimensions or ordering.
;
;Keywords:
;  subtract:  If this keyword is set, this function will subtract leap seconds from a date that already includes them.
;  
;  leap_seconds: Set this to a named variable in which to return the offsets used for the calculation.  Note that these can differ in both sign and magnitude for
;  a particular date, depending on whether you are adding or subtracting.
;
;Return Value:
;  The data with leap seconds added(or removed). Will preserve double or string data types.
;  (e.g. If you pass in a string, it returns a string)
;  
;Notes:
;  #1 This routine requires the CDF 3.4 leap second file.  One will be provided with the TDAS install, but it will be updated
;  automatically when a new leap second occurs.
;  
;  #2 The environment variable "!CDF_LEAP_SECONDS" must be defined for this routine to work.  This variable is defined by
;  calling cdf_leap_second_init.  Normally, the initialization routine for missions that use tt2000 should use this.
;
;Examples:
;THEMIS>   print,add_leap_seconds('2007-03-23')
;2007-03-23/00:00:33
;THEMIS>   print,add_leap_seconds('2007-03-23',/subtract)
;2007-03-22/23:59:27
;THEMIS>   print,add_leap_seconds(add_leap_seconds('2007-03-23'),/subtract)
;2007-03-23/00:00:00
;THEMIS>   print,add_leap_seconds(time_double(['2007-03-23','1980-01-01/23:59:59']))
;   1.1746080e+09   3.1561922e+08
;THEMIS>   print,add_leap_seconds(['2007-03-23','1980-01-01/23:59:59'],leap_seconds=ls_offset)
;2007-03-23/00:00:33 1980-01-02/00:00:18
;THEMIS>   print,ls_offset
;      33.0000      19.0000
;
; $LastChangedBy: pcruce $
; $LastChangedDate: 2012-04-10 11:18:55 -0700 (Tue, 10 Apr 2012) $
; $LastChangedRevision: 10312 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/CDF/TT2000/add_leap_seconds.pro $
;-

function add_leap_seconds,dates,subtract=subtract,leap_seconds=leap_seconds

  double_data=time_double(dates)

  defsysv,'!CDF_LEAP_SECONDS',exists=exists
  
  if ~keyword_set(exists) then begin
    ;fatal error
    message,'Error. !CDF_LEAP_SECONDS, must be defined.  Try calling cdf_leap_second_init'
  endif
  
  leap_data = read_asc(!cdf_leap_seconds.local_data_dir+'/CDFLeapSeconds.txt')
  leap_dates= time_double(strtrim(leap_data.(0),2)+'-'+strtrim(leap_data.(1),2)+'-'+strtrim(leap_data.(2),2)+'/00:00:00')
  leap_offsets=leap_data.(3)
  
  if keyword_set(subtract) then begin
    leap_dates+=leap_offsets
  endif 
  
  ;finds which offset parameters apply to which input dates
  offset_indexes=value_locate(leap_dates,double_data,/l64) ;this assumes that leap_dates is monotone
  leap_seconds=([0,leap_offsets])[offset_indexes+1] ;offset_indexes has range [-1,n-1], so prepending the zero offset element allows indexing without error 
  
  if keyword_set(subtract) then begin
    leap_seconds*=-1
  endif
  
  double_data+=leap_seconds
 
  if is_string(dates) then begin
    return,time_string(double_data)
  endif else begin
    return,double_data
  endelse

end